#!/bin/sh
#set -x

# AIX ODM VPD cleaner - clean entries from the ODM VPD related to corrupt
# WebSphere Application Server installations.
#
# Here is a quick primer on the ODM and what the cleaning entails:
# Starting in AIX 5.1, the ODM is made of 5 classes:
#   Inventory
#   LPP
#   History
#   Product
#   Vendor
#
# The WebSphere Application Server (WSAS) installer writes entries to each of
# those classes.  In order to cleanly re-install WSAS to a specific directory,
# the ODM must be cleaned of all references to that directory.  The ODM is
# normally cleaned by the WSAS uninstall program.  If the uninstaller is
# unavailable, then the ODM must be cleaned manually.  If the ODM is not
# cleaned, then the product will not successfully re-install to the "old"
# directory.  If the ODM is improperly cleaned, then the installer will throw
# errors.
#
# Each ODM class can be queried based on various criteria.  In order to clean
# the ODM, certain classes (such as inventory and vendor) are queried based
# on the installation directory.  The information obtained from those queries
# can be used in queries to other classes in the ODM.  As a result, criteria
# matching all of the ODM entries relating to a product's installation are
# obtained.
#
# This script performs queries based on the product's installation directory
# and generates a script based on the results.  The generated script's purpose
# is to delete all the ODM entries associated with a product's installation.
#
# Note that this script uses odmdel as opposed to vpddel in order to excercise
# more granulated control in situations where the product information in the
# ODM could be incomplete.


# An ODM cleanup entails these actions.  PRODDIR = Product install directory.
#  Gather info:
#  1.  Get installation directory from user input, store it in PRODDIR
#  2a. List ODM inventory, grep for all occurrences of PRODDIR, store list of
#      files in INV_LOC_BY_PRODDIR.  Remove entries listed in EXCLUDE_DIR.
#  2b. Iterate through INV_LOC_BY_PRODDIR, querying ODM LPP for each file.  
#      Store list of unique LPP IDs in INV_LPPID_BY_PRODDIR.
#  2c. Iterate through INV_LPPID_BY_PRODDIR, querying ODM inventory for each lpp
#      id.  If a loc0 entry appears which does NOT contain PRODDIR or does
#      contain an EXCLUDE_DIR entry, then place lpp value in LPPID_SHARE_LIST.
#  3a. List ODM vendor, grep for all occurrences of PRODDIR, store list of
#      matching ddir fields in VENDOR_DDIR_BY_PRODDIR.  Remove entries
#      listed in EXCLUDE_DIR.
#  3b. Query ODM vendor based on VENDOR_DDIR_BY_PRODDIR, extract ddir, GUID, 
#      lpp_id, time, and instance, delimited by | character, into
#      VENDOR_INFO_BY_DDIR. Remove ddir entries listed in EXCLUDE_DIR.
#  3c. Query ODM vendor based on GUID from VENDOR_INFO_BY_DDIR.  If a ddir
#      entry appears which does NOT match PRODDIR or does match an EXCLUDE_DIR
#      entry, then place GUID value in GUID_SHARE_LIST.
#  4a. Iterate through INV_LPPID_BY_PRODDIR, extract entries which do NOT appear
#      in LPPID_SHARE_LIST and place them in LPPID_DELETE_LIST.
#  4b. Iterate through VENDOR_INFO_BY_DDIR, extract GUID entries which do NOT
#      appear in GUID_SHARE_LIST and place them in GUID_DELETE_LIST.
#  
#  Select entries for deletion:
#  inventory:
#    loc0 = each entry in INV_LOC_BY_PRODDIR
#  lpp:
#    lpp_id = each entry in LPPID_DELETE
#  product:
#    lpp_name = each entry in GUID_DELETE
#  history:
#    lpp_id = each lpp_id entry from VENDOR_INFO_BY_DDIR
#    time = each corresponding time entry from VENDOR_INFO_BY_DDIR
#  vendor:
#    ddir = each ddir entry in VENDOR_DDIR_BY_PRODDIR
#  vpd.properties file:
#    first entry = each GUID from VENDOR_INFO_BY_DIR
#    last entry = each cooresponding instance from VENDOR_INFO_BY_DIR
#


# Examples for usage of "awk" in this script:
#    awk ' (NR==3) {print $3}'
#      Extract the 3rd line (NR==3) of data, and from that extract the 3rd
#      "column" of data.  So, processing this line:
#          name = "WSBJD9AA"
#      would return (including the quotes):
#          "WSBJD9AA"
#
#    awk -F'\"' '{print $2}'
#      This is one method of stripping quotes off a "simple" quoted value.
#      So, processing this line:
#          "WSBJD9AA"
#      would return:
#          WSBJD9AA
#




main() {
    echo " "

    init_var
    check_util
    test_write
    write_greeting
    get_PRODDIR
    prep_log
    detect_exclude
    
    get_inv_loc_by_proddir
    get_inv_lppid_by_proddir
    get_share_inv_lppid
    
    get_vendor_ddir_by_proddir
    get_vendor_info_by_ddir
    get_share_vendor_guid

    get_lppid_delete
    get_guid_delete
    #get_vpdproperties_delete
    
    script_generate_header
    script_product_delete
    script_history_delete
    script_lpp_delete
    script_inventory_delete
    script_vendor_delete
    #script_vpdproperties_delete
    #script_generate_footer
}




# init_var - Initialize global variables

init_var() {
    export ODMDIR=/usr/lib/objrepos
    VPD_PROPERTIES_NAME=vpd.properties
    VPD_PROPERTIES_FILE=$ODMDIR/$VPD_PROPERTIES_NAME
    ODMFIELDS_UTIL=./odmfields.awk
    
    LOGFILE=./generateODMclean.log
    SCRIPTFILE=./manualODMclean.sh

    EXCLUDE_DIR=./exclude.data


    TESTFILE=./testfile
    TMPFILE=./tmpfile.out
    SWAPFILE=./swap.out

    INV_LOC_BY_PRODDIR=./inv_loc_proddir.list
    INV_LPPID_BY_PRODDIR=./inv_lppid_loc.list
    INV_LPPID_SHARE=./inv_lppid_share.list
    LPPID_DELETE=./lppid_delete.list

    VENDOR_DDIR_BY_PRODDIR=./vendor_ddir_proddir.list
    VENDOR_INFO_BY_DDIR=./vendor_info.list
    VENDOR_GUID_SHARE=./vendor_guid_share.list
    GUID_DELETE=./guid_delete.list
}




# test_write - Test ability to write to current directory.  Die if test fails.

test_write() {
    echo Date: `date` >$TESTFILE
    if [[ -f $TESTFILE ]] then
        rm $TESTFILE
    else
        echo "Error: Cannot write to current directory.  Ensure $(whoami) has write access to:"
        pwd
        echo " "
        exit 1
    fi
}




# check_util - Check that the ODMFIELDS awk utility is present.  Die if check fails.

check_util() {
    if [[ ! -f $ODMFIELDS_UTIL ]] then
        echo "Error:  $ODMFIELDS_UTIL not found.  Please ensure that that file is in the"
        echo "current directory and try again."
        echo " "
        exit 1
    fi
}




# detect_exclude - Detect if the "exclude" file is present.  If so, set EXCLUDE=1.
# Dump the content of the exclude file to the log.
    
detect_exclude() {
    if [[ -f $EXCLUDE_DIR ]] then
        EXCLUDE=1
        echo "-------- Exclude file detected: $EXCLUDE_DIR." | tee -a $LOGFILE
        echo "Directores in this listing will be filtered from processing:" | tee -a $LOGFILE
        cat $EXCLUDE_DIR | tee -a $LOGFILE
        echo "---- end of listing" | tee -a $LOGFILE
    else
        EXCLUDE=0
        echo "No exclude file detected." >> $LOGFILE
        
        # A more user-friendly message is printed to console
        echo "All products in or below input directory will be processed."
        echo " "
    fi
}



# prep_log - Append header to log file

prep_log() {
    >$LOGFILE
    echo "---------------- begin manualODMclean log" >>$LOGFILE
    echo "$(date)" >>$LOGFILE
    echo "Current directory: $PWD" >>$LOGFILE
    echo "Command line: $*" >>$LOGFILE
    echo " " >>$LOGFILE
    echo "Current user: `whoami`" >>$LOGFILE
    echo "Operating on user input: $PRODDIR" >>$LOGFILE
    echo " " >>$LOGFILE
}




#  List ODM inventory, grep for all occurrences of PRODDIR, store list of
#  files in INV_LOC_BY_PRODDIR
#
# Retrieve entire ODM inventory.
# Grep for entries starting with "$PRODDIR(/|")
# Literally, find double-quote, product directory, and slash OR double-quote.
# This specifically targets an exact directory name match ("$PRODDIR") or
# subdirectories of that directory ("$PRODDIR/"), but NOT similarly-named
# directories not in that structure.  For example, PRODDIR = /usr/mydir
# will match "/usr/mydir" and "/usr/mydir/anotherdir", but NOT "/usr/mydir2"


get_inv_loc_by_proddir() {
    # Print header

    echo " " | tee -a $LOGFILE
    echo "-------- Retrieving file locations from inventory based on input" | tee -a $LOGFILE
    echo "This may take some time.  Temporary output is written to $INV_LOC_BY_PRODDIR."
    
    
    # Generate entries

    #Get entries starting with the directory name, followed by a slash and anything else
    INV_LOC_COUNT=`odmget -q "loc0 LIKE '$PRODDIR/*'" inventory 2>>$LOGFILE | grep loc0 2>>$LOGFILE | awk -F'\"' '{print $2}' | tee $INV_LOC_BY_PRODDIR | wc -l`
    INV_LOC_COMPARE=`cat $INV_LOC_BY_PRODDIR | wc -l`
    
    # If the number of items returned from the ODM query do not match the number of items
    # written to the file, then the filesystem may have filled up during this process.
    # Warn user of this condition.

    if [[ $INV_LOC_COUNT != $INV_LOC_COMPARE ]] then
      echo "Error:  The content of $INV_LOC_COUNT appears incomplete.  Is the filesystem full?"
      exit 1
    fi
    
    # Append entries that are exactly the directory name (which wouldn't be matched above.)
    odmget -q "loc0 = '$PRODDIR'" inventory 2>>$LOGFILE | grep loc0 2>>$LOGFILE | awk -F'\"' '{print $2}' >>$INV_LOC_BY_PRODDIR


    # Filter entries based on content of EXCLUDE_DIR
    
    if (( $EXCLUDE == 1 )) then
        for exc in `cat $EXCLUDE_DIR`
        do
            cat $INV_LOC_BY_PRODDIR | grep -v \^$exc >$TMPFILE
            mv $TMPFILE $INV_LOC_BY_PRODDIR
        done
    fi
    

    # Warn if no inventory entries are found.
    
    INV_LOC_COUNT=`cat $INV_LOC_BY_PRODDIR | wc -l`
    echo "---- Found $INV_LOC_COUNT inventory entries" | tee -a $LOGFILE
    if (( $INV_LOC_COUNT == 0 )) then
        inv_loc_count_warning
    else
        echo " "
        echo "---- Here are the first 10 of the $INV_LOC_COUNT entries for your verification"
      #TODO: is this verifcation necessary?  if so, instruct what to do
        head $INV_LOC_BY_PRODDIR
    fi
    
    cat $INV_LOC_BY_PRODDIR >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




inv_loc_count_warning() {
    echo " "
    echo "Warning: No inventory entries found containing references to"
    echo "\"$PRODDIR\/"
    echo " "
    echo "This could indicate that the directory name was entered incorrectly, but"
    echo "it is also possible that these entries have somehow already been removed"
    echo "from the ODM.  If you are absolutely sure that the directory name has been"
    echo "entered correctly, then proceed."
    echo "Press ENTER to continue, or use ^C to terminate this script now."
    read CONTINUE
}


#  List ODM vendor, grep for all occurrences of PRODDIR, store list of
#  matching ddir fields in VENDOR_DDIR_BY_PRODDIR
#
# Retrieve entire ODM vendor.
# Grep for entries starting with ddir = "$PRODDIR(/|")
# Literally, after identifying a ddir field, find double-quote, 
# product directory, and slash OR double-quote.
# This specifically targets an exact directory name match ("$PRODDIR") or
# subdirectories of that directory ("$PRODDIR/"), but NOT similarly-named
# directories not in that structure.  For example, PRODDIR = /usr/mydir
# will match "/usr/mydir" and "/usr/mydir/anotherdir", but NOT "/usr/mydir2"


get_vendor_ddir_by_proddir() {

    # Print header
    
    echo " " | tee -a $LOGFILE
    echo "-------- Retrieving ddir entries from vendor based on input" | tee -a $LOGFILE
    
    
    # Generate entries
    
    >$TMPFILE
    
    # Get exact matches
    odmget -q "ddir = $PRODDIR" vendor | grep ddir | awk -F'\"' '{print $2}' >$TMPFILE 2>>$LOGFILE
    
    # Get subdirectories of exact match
    odmget -q "ddir LIKE '$PRODDIR/*'" vendor | grep ddir | awk -F'\"' '{print $2}' >>$TMPFILE 2>>$LOGFILE
    
    cat $TMPFILE | sort -u >$VENDOR_DDIR_BY_PRODDIR
    
    # Filter entries based on content of EXCLUDE_DIR
    
    if (( $EXCLUDE == 1 )) then
      for exc in `cat $EXCLUDE_DIR`
      do
        cat $VENDOR_DDIR_BY_PRODDIR | grep -v \^$exc >$TMPFILE
        mv $TMPFILE $VENDOR_DDIR_BY_PRODDIR
      done
    fi
    
    
    # Print count and log footer
    
    VENDOR_DDIR_COUNT=`cat $VENDOR_DDIR_BY_PRODDIR | wc -l`
    echo "---- Found $VENDOR_DDIR_COUNT unique ddir entries in vendor" | tee -a $LOGFILE
    cat $VENDOR_DDIR_BY_PRODDIR >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




#  Iterate through INV_LOC_BY_PRODDIR, querying ODM LPP for each file.  
#  Store list of unique LPP IDs in INV_LPPID_BY_PRODDIR.


get_inv_lppid_by_proddir() {
    # Print header
    
    echo " " | tee -a $LOGFILE
    echo "-------- Retrieving LPP IDs from inventory" | tee -a $LOGFILE
    

    # Begin processing the loc0 entries in the background.  Check every 4 seconds
    # for finish.  Update console with status message every 20 seconds.
    
    loop_inv_lpp_by_proddir &
    echo "This may take some time. Running in background process ID $!."
    echo "If this script is terminated, be sure to also kill that process."
    echo "Temporary output is written to $TMPFILE"
    wait_for_inv_lpp_loop


    # Save entries in data file and write log footer
    
    cat $TMPFILE | grep -v \|\|\|FINISH | sort -u >$INV_LPPID_BY_PRODDIR
    
    echo "---- Found $(cat $INV_LPPID_BY_PRODDIR | wc -l) unique LPP entries" | tee -a $LOGFILE
    cat $INV_LPPID_BY_PRODDIR >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




# Loop through inventory locations, extract lpp entries
# Note that the lpp id fields cannot be retrieved by a simple query because of
# the possibility that some directories are excluded from the listing resulting
# from a simple query.
# Loop iterates until all entries are processed, then appends "|||FINISH" to the
# end of the TMPFILE.  This string should be detected by the parent process
# monitoring the TMPFILE.

loop_inv_lpp_by_proddir() {
    >$TMPFILE
    for loc in `cat $INV_LOC_BY_PRODDIR`
    do
        odmget -q "loc0 = $loc" inventory | awk ' (NR==3) {print $3}' >>$TMPFILE 2>>$LOGFILE
    done
    echo \|\|\|FINISH >>$TMPFILE
}




# Check status of lpp file

check_loop_inv_lpp_status() {
    if [[ x`cat $TMPFILE | grep \^\|\|\|FINISH` = x\|\|\|FINISH ]] then
        return 0
    else
        return 1
    fi
}



# Check every 4 seconds for finish.  Update console with status message every 20 seconds.

wait_for_inv_lpp_loop() {
    # Give the external loop a chance to warm up, so sleep 1, test, then start loop
    sleep 1
    check_loop_inv_lpp_status
    if (( $? == 0 )) then 
        return 0
    fi

    while true
    do
        for i in 1 2 3 4 5 6
        do
            # Sleep for 5 seconds then check to see if child process loop has terminated
            
            sleep 5
            check_loop_inv_lpp_status
            if (( $? == 0 )) then 
                break 2
            fi
        done
        
        
        # Approximately 30 seconds have elapsed, so print status message
        
        echo "Processed `cat $TMPFILE | wc -l` of $INV_LOC_COUNT entries"
    done
}




#  Iterate through INV_LPPID_BY_PRODDIR, querying ODM inventory for each lpp
#  id.  If a loc0 entry appears which does NOT contain PRODDIR or an excluded
#  entry, then place lpp value in LPPID_SHARE_LIST.

get_share_inv_lppid() {
    # Print header

    echo " " | tee -a $LOGFILE
    if (( $EXCLUDE == 1 )) then
        echo "-------- Cross-checing loc0 fields and excluded locations with lpp_id" | tee -a $LOGFILE
    else
        echo "-------- Cross-checing loc0 fields with lpp_id" | tee -a $LOGFILE
    fi

    # Loop through lpp_id entries.  If a loc0 entry does NOT contain PRODDIR
    # or if the entry matches an entry on the exclude list, then save the
    # current lpp_id in the share list.
    # This identifies lpp id entries shared by other products.
    
    >$SWAPFILE
    >$INV_LPPID_SHARE
    for lpp in `cat $INV_LPPID_BY_PRODDIR`
    do
        # Generate list of loc0 fields which are associated with this lppid
        odmget -q "lpp_id = $lpp" inventory | grep "loc0 = \"" | awk -F'\"' '{print $2}' >$TMPFILE 2>>$LOGFILE
        echo "-- lpp_id $lpp has `cat $TMPFILE | wc -l` loc0 references." >>$LOGFILE
        
        # Throw away exact PRODDIR matches so the next test will work properly
        cat $TMPFILE | grep -v \^$PRODDIR\$ >>$SWAPFILE
        mv $SWAPFILE $TMPFILE
        
        # Count entries which do not contain initial substring PRODDIR/
        # If count is not zero, then record this lpp_id in the share list and restart loop
        NO_PRODDIR_COUNT=`cat $TMPFILE | grep -v \^$PRODDIR/ | wc -l`
        
        if (( $NO_PRODDIR_COUNT != 0 )) then
            echo $lpp >>$INV_LPPID_SHARE
            
            echo "TRUE: $NO_PRODDIR_COUNT of those entries are not related to target directory" >>$LOGFILE
        else
            # Check each exclude entry against the loc0 entry list
            if (( $EXCLUDE == 1 )) then
                for exc in `cat $EXCLUDE_DIR`
                do
                    # Search loc0 entry list for excluded entry
                    EXCLUDED_COUNT=`cat $TMPFILE | grep \^$exc | wc -l`
                    if (( $EXCLUDED_COUNT != 0 )) then
                        echo $lpp >>$INV_LPPID_SHARE
                        
                        echo "TRUE: $EXCLUDED_COUNT of those entries are the excluded entry $exc" >>$LOGFILE
                        break
                    fi
                done
                if (( $EXCLUDED_COUNT == 0 )) then
                    echo "FALSE: No other inventory entries are associated with this lpp_id." >>$LOGFILE
                fi
            else
                echo "FALSE: No other inventory entries are associated with this lpp_id." >>$LOGFILE
            fi
        fi
    done
    
    
    echo "---- Found $(cat $INV_LPPID_SHARE | wc -l) of this product's lpp_id entries used by other products" | tee -a $LOGFILE
    if (( $EXCLUDE == 1 )) then
        echo "     or excluded entries" | tee -a $LOGFILE
    fi
    
    cat $INV_LPPID_SHARE >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




#  Query ODM vendor based on PRODDIR, extract ddir, GUID, lpp_id, time, and
#  instance, delimited by | character, into VENDOR_INFO_BY_DDIR.


get_vendor_info_by_ddir() {
    # Print header

    echo " " | tee -a $LOGFILE
    echo "-------- Retrieving full information from vendor" | tee -a $LOGFILE
    

    # Query vendor based on PRODDIR, extract entries into delimted fields using awk script
    
    >$VENDOR_INFO_BY_DDIR
    for ddir_entry in `cat $VENDOR_DDIR_BY_PRODDIR`
    do
        >$TMPFILE
        odmget -q "ddir = $ddir_entry" vendor >>$TMPFILE
        cat $TMPFILE | awk -f $ODMFIELDS_UTIL >>$VENDOR_INFO_BY_DDIR 2>>$LOGFILE
    done
    

    # Save entries in data file and write log footer

    echo "---- Processed $(cat $VENDOR_INFO_BY_DDIR | wc -l) entries" | tee -a $LOGFILE
    cat $VENDOR_INFO_BY_DDIR >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




#  Query ODM vendor based on GUID from VENDOR_INFO_BY_DDIR.  If a ddir
#  entry appears which does NOT match PRODDIR, then place GUID value in
#  VENDOR_GUID_SHARE.


get_share_vendor_guid() {
    # Print header
    echo " " | tee -a $LOGFILE
    echo "-------- Cross-checking GUID entries against ddir" | tee -a $LOGFILE


    # Iterate through guid entries extracted from vendor, extracting guid fields
    # which are associated with ddir fields which do NOT match PRODDIR.
    # This identifies the guid entries which are shared by other installations.

    >$VENDOR_GUID_SHARE
    for RAW_GUID_ENTRY in `cat $VENDOR_INFO_BY_DDIR`
    do
        # Extract the second field (GUID) from this entry
        guidentry=`echo $RAW_GUID_ENTRY | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\2/'`
      
        # Query vendor for exact matches on GUID and examine resulting ddir entries.
        # Filter ddir entries which exactly match the RAW_GUID_ENTRY's first field (ddir)
        odmget -q "GUID = $guidentry" vendor 2>>$LOGFILE | grep -E "ddir = \"" 2>>$LOGFILE | awk -F'\"' '{print $2}' | grep -v \^$PRODDIR\$ 2>>$LOGFILE >$TMPFILE
        echo "-- GUID  $guidentry has $NO_PRODDIR_COUNT vendor references which don't exactly match input directory." >>$LOGFILE
        
        # Count entries which do not contain initial substring PRODDIR/
        # If count is not zero, then record this guid in the share list and restart loop
        NO_PRODDIR_COUNT=`cat $TMPFILE | grep -v \^$PRODDIR/ | wc -l`
        
        if (( $NO_PRODDIR_COUNT != 0 )) then
            echo $guidentry >>$VENDOR_GUID_SHARE
            
            echo "TRUE: $NO_PRODDIR_COUNT of those entries are not related to target directory" >>$LOGFILE
        else
            # Check each exclude entry against the ddir entry list
            if (( $EXCLUDE == 1 )) then
                for exc in `cat $EXCLUDE_DIR`
                do
                    # Search ddir entry list for excluded entry
                    EXCLUDED_COUNT=`cat $TMPFILE | grep \^$exc | wc -l`
                    if (( $EXCLUDED_COUNT != 0 )) then
                        echo $guidentry >>$VENDOR_GUID_SHARE
                        
                        echo "TRUE: $EXCLUDED_COUNT of those entries are the excluded entry $exc" >>$LOGFILE
                        break
                    fi
                done
                if (( $EXCLUDED_COUNT == 0 )) then
                    echo "FALSE: No other vendor entries are associated with this GUID." >>$LOGFILE
                fi
            else
                echo "FALSE: No other vendor entries are associated with this GUID." >>$LOGFILE
            fi
        fi
    done
    
    
    # Save entries in data file and write log footer

    cat $VENDOR_GUID_SHARE | sort -u >$SWAPFILE
    mv $SWAPFILE $VENDOR_GUID_SHARE
    echo "---- Found $(cat $VENDOR_GUID_SHARE | wc -l) of this product's GUID entries referenced by other products" | tee -a $LOGFILE
    cat $VENDOR_GUID_SHARE >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




#  Iterate through INV_LPPID_BY_PRODDIR, extract lpp_id entries which do NOT appear
#  in LPPID_SHARE_LIST and place them in LPPID_DELETE.

get_lppid_delete() {
     # Print header
    
    echo " " | tee -a $LOGFILE
    echo "-------- Selecting lpp_id entries unique to this installation" | tee -a $LOGFILE
    
    
    # Iterate through lpp id entries discovered during inventory query and extract
    # entries which do not appear in the list of shared lpp id entries
    >$TMPFILE
    for lpp_id in `cat $INV_LPPID_BY_PRODDIR`
    do
        RESULT_COUNT=`cat $INV_LPPID_SHARE | grep \^$lpp_id\$ | wc -l`
        if (( RESULT_COUNT == 0 )) then
             echo $lpp_id >>$TMPFILE
        fi
    done
    
    
    # Save entries in data file and write log footer

    cat $TMPFILE | sort -u >$LPPID_DELETE
    echo "---- Found `cat $LPPID_DELETE | wc -l` lpp_id entries targeted for deletion" | tee -a $LOGFILE
    cat $LPPID_DELETE >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




#  Iterate through VENDOR_INFO_BY_DDIR, extract GUID entries which do NOT
#  appear in VENDOR_GUID_SHARE and place them in GUID_DELETE.

get_guid_delete() {
    # Print header

    echo " " | tee -a $LOGFILE
    echo "-------- Selecting GUID entries unique to this installation" | tee -a $LOGFILE
    
    
    # Iterate through guid entries discovered during vendor query and extract
    # entries which do not appear in the list of shared guid entries

    >$TMPFILE
    for RAW_GUID_ENTRY in `cat $VENDOR_INFO_BY_DDIR`
    do
        # Extract the first field (GUID) from this entry
        guid=`echo $RAW_GUID_ENTRY | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\2/'`
        
        RESULT_COUNT=`cat $VENDOR_GUID_SHARE | grep \^$guid\$ | wc -l`
        if (( RESULT_COUNT == 0 )) then
            echo $guid >>$TMPFILE
        fi
    done
    
    
    # Save entries in data file and write log footer
    
    cat $TMPFILE | sort -u >$GUID_DELETE
    echo "---- Found $(cat $GUID_DELETE | wc -l) guid entries targeted for deletion" | tee -a $LOGFILE
    cat $GUID_DELETE >>$LOGFILE
    echo "---- end of listing" >>$LOGFILE
}




# Warn if no vendor entries are found.
# If no inventory and no vendor entries were found, exit.  This script
# cannot do anything if it cannot find references to PRODDIR in the ODM.
    #TODO: Extend to account for vpd.properties cleaning.
    
check_relevant_findings() {
    if (( $VENDOR_DDIR_COUNT == 0 -a $INV_LOC_COUNT != 0)) then
      echo "Warning: No vendor entries found containing references to"
      echo "\"$PRODDIR\/"
      echo " "
      echo "This could indicate that the directory name was entered incorrectly, but"
      echo "it is also possible that these entries have somehow already been removed"
      echo "from the ODM.  If you are absolutely sure that the directory name has been"
      echo "entered correctly, then proceed."
      echo "Press ENTER to continue, or use ^C to terminate this script now."
      read CONTINUE
    else
      if (( $VENDOR_DDIR_COUNT == 0 -a $INV_LOC_COUNT == 0)) then
        echo "Error:  No vendor entries found containing references to"
        echo "\"$PRODDIR\/"
        echo " "
        echo "As previously indicated, no matching inventory entries were found, either."
        echo "This script cannot continue because it is unable to find references to the"
        echo "specified directory in the ODM VPD."
        echo " "
        echo "This could mean that one of these conditions is true:"
        echo "- An incorrect directory was specified."
        echo "- Some or all references to the product have already been removed from the ODM VPD."
        echo " "
        echo "If there is a chance that the product was properly cleaned up before running this"
        echo "script, then it is reasonable to conclude that the product has been removed from"
        echo "the ODM VPD, and that it is safe to install a new instance of the product."
        echo 
      fi
    fi
}




################################
#  Generate cleaning script

script_generate_header() {
    echo " " | tee -a $LOGFILE
    echo "-------- Generating custom cleanup script named $SCRIPTFILE" | tee -a $LOGFILE
    SCRIPTDATE=`date`
    >$SCRIPTFILE
    
    echo '#!/bin/sh'                                                                             >$SCRIPTFILE
    echo ' '                                                                                     >>$SCRIPTFILE
    echo 'SCRIPTDATE='\'$SCRIPTDATE\'                                                            >>$SCRIPTFILE
    echo 'export ODMDIR='$ODMDIR                                                                 >>$SCRIPTFILE
    echo ' '                                                                                     >>$SCRIPTFILE
    echo '# Custom cleanup script generated by AIX ODM VPD cleanup script.'                      >>$SCRIPTFILE
    echo '# Script generated on this date:'                                                      >>$SCRIPTFILE
    echo '#  ' $SCRIPTDATE                                                                       >>$SCRIPTFILE
    echo '# The purpose of this script is to clean entries out of the ODM VPD related to'        >>$SCRIPTFILE
    echo '# this installation directory, without disturbing ODM entries in use by other'         >>$SCRIPTFILE
    echo '# products:'                                                                           >>$SCRIPTFILE
    echo '#  ' $PRODDIR                                                                          >>$SCRIPTFILE
    echo '#'                                                                                     >>$SCRIPTFILE
    echo ' '                                                                                     >>$SCRIPTFILE
    echo ' '	                                                                                 >>$SCRIPTFILE
    echo 'echo "This script will clean out the AIX ODM registry from a partially-uninstalled"'   >>$SCRIPTFILE
    echo 'echo "or corrupt WebSphere Application Server product."'                               >>$SCRIPTFILE
    echo 'echo " "'                                                                              >>$SCRIPTFILE
    echo 'echo "The script was instructed to delete entries related to this installation"'       >>$SCRIPTFILE
    echo 'echo "directory:"'                                                                     >>$SCRIPTFILE
    echo 'echo "$PRODDIR"'                                                                       >>$SCRIPTFILE
    
    if (( EXCLUDE == 1 )) then
        echo 'echo " "' >>$SCRIPTFILE
        echo 'echo "The script was instructed to specifically avoid these installation directories:"' >>$SCRIPTFILE
        for ent in `cat $EXCLUDE_DIR`
        do
           echo 'echo "'$ent'"' >>$SCRIPTFILE
        done
    fi
    
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "It is important to attempt to uninstall the targeted product installation using"'  >>$SCRIPTFILE
    echo 'echo "the uninstaller program before running this script."'                              >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "As a standard precaution, the system should be backed up beforehand."'             >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "Please note that if you are manually uninstalling a WebSphere Application"'        >>$SCRIPTFILE
    echo 'echo "Server product, you must follow the manual uninstall instructions outlined in"'    >>$SCRIPTFILE
    echo 'echo "the WebSphere InfoCenter. This script is provided to assist with that process."'   >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "This script was generated on $SCRIPTDATE"'                                         >>$SCRIPTFILE
    echo 'echo "It is NOT recommended to use this script if it is more than a few days old. This"' >>$SCRIPTFILE
    echo 'echo "script is invalidated if any other products have been installed or uninstalled"'   >>$SCRIPTFILE
    echo 'echo "since the time this script was generated."'                                        >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "All output from this script is generated on the console.  If it is necessary"'     >>$SCRIPTFILE
    echo 'echo "to capture output, you must do so by using a file redirect when starting this"'    >>$SCRIPTFILE
    echo 'echo "script."'                                                                          >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo 'echo "Press ENTER to continue, or use ^C to quit now."'                                  >>$SCRIPTFILE
    echo 'read CONTINUE'                                                                           >>$SCRIPTFILE
    echo 'echo " "'                                                                                >>$SCRIPTFILE
    echo ' '                                                                                       >>$SCRIPTFILE
}




script_product_delete() {
    TARGET_CLASS='product class'
    TARGET_COUNT=`cat $GUID_DELETE | wc -l | awk '{print $1}'`
    
    script_print_class_header
    
    for ent in `cat $GUID_DELETE`
    do
        echo 'odmdelete -o product -q "lpp_name = '$ent'"' >>$SCRIPTFILE
    done
}





script_history_delete() {
    TARGET_CLASS='history class'
    TARGET_COUNT=`cat $VENDOR_INFO_BY_DDIR | wc -l | awk '{print $1}'`
    
    script_print_class_header
    
    for ent in `cat $VENDOR_INFO_BY_DDIR`
    do
        lpp_id_field=`echo $ent | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\4/'`
        time_field=`echo $ent | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\5/'`
        echo 'odmdelete -o history -q "lpp_id = '$lpp_id_field' AND time = '$time_field'"' >>$SCRIPTFILE
    done
}





script_lpp_delete() {
    TARGET_CLASS='lpp class'
    TARGET_COUNT=`cat $LPPID_DELETE | wc -l | awk '{print $1}'`
    
    script_print_class_header
    
    for ent in `cat $LPPID_DELETE`
    do
        echo 'odmdelete -o lpp -q "lpp_id = '$ent'"' >>$SCRIPTFILE
    done
}





script_inventory_delete() {
    TARGET_CLASS='inventory class'
    TARGET_COUNT=`cat $INV_LOC_BY_PRODDIR | wc -l | awk '{print $1}'`
    
    script_print_class_header
    
    for ent in `cat $INV_LOC_BY_PRODDIR`
    do
        echo 'odmdelete -o inventory -q "loc0 = '$ent'"' >>$SCRIPTFILE
    done
}





script_vendor_delete() {
    TARGET_CLASS=vendor
    TARGET_COUNT=`cat $VENDOR_DDIR_BY_PRODDIR | wc -l | awk '{print $1}'`
    
    script_print_class_header
    
    for ent in `cat $VENDOR_DDIR_BY_PRODDIR`
    do
        echo 'odmdelete -o vendor -q "ddir = '$ent'"' >>$SCRIPTFILE
    done
}





script_vpdproperties_delete() {
# needs completion
    TARGET_CLASS='vpd.properties file'
    TARGET_COUNT=`cat $VENDOR_INFO_BY_DDIR | wc -l | awk '{print $1}'`
    
    script_print_class_header
    #backup_vpd_properties
    #copy_vpd_properties
    
    for ent in `cat $VENDOR_INFO_BY_DDIR`
    do
        guid_entry=`echo $ent | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\2/'`
        instance_entry=`echo $ent | sed 's/^\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)|\([^|]*\)$/\3/'`
        echo "vpd.properties - Delete guid: $guid_entry   instance: $instance_entry"
    done
}





script_generate_footer() {
    echo " "                                                                            >>$SCRIPTFILE
    echo "Process complete."                                                            >>$SCRIPTFILE
}




script_print_class_header() {
    echo " "                                                                            >>$SCRIPTFILE
    echo " "                                                                            >>$SCRIPTFILE
    echo " "                                                                            >>$SCRIPTFILE
    echo " "                                                                            >>$SCRIPTFILE
    echo "################################"                                             >>$SCRIPTFILE
    echo "################################"                                             >>$SCRIPTFILE
    echo "#"                                                                            >>$SCRIPTFILE
    echo "# Delete entries from $TARGET_CLASS"                                          >>$SCRIPTFILE
    echo "#"                                                                            >>$SCRIPTFILE
    echo "################################"                                             >>$SCRIPTFILE
    echo "################################"                                             >>$SCRIPTFILE
    echo " "                                                                            >>$SCRIPTFILE
    echo 'echo "-------- Deleting' $TARGET_COUNT 'entries from' $TARGET_CLASS '"'       >>$SCRIPTFILE

    echo "Generating $TARGET_COUNT entries for $TARGET_CLASS" | tee -a $LOGFILE
}




################################
#  Output information


# write_greeting - Writes the opening greeting and warnings.

write_greeting() {
    echo "This utility will generate a script which will clean entries related to a"
    echo "partially-uninstalled or corrupt WebSphere Application Server product from"
    echo "the AIX ODM VPD. Note that the utility running now will not actually alter the"
    echo "ODM. The script resulting from this utility is responsible for making changes."
    echo " "
    echo "This script may generate several megabytes of data.  Please make sure at least"
    echo "10 megabytes are available in the current filesystem."
    echo " "
    echo "Use this utility with care! The resulting script will delete ALL ODM VPD"
    echo "entries which are associated with the input directory as well as all the"
    echo "subdirectories. If another product is installed under the same directory as"
    echo "specified, then that product will be removed as well."
    echo " "
    echo "It is important to attempt to uninstall the targeted product installation using"
    echo "the uninstaller program before using this utility."
    echo " "
    echo "As a standard precaution, the system should be backed up before running the"
    echo "script produced by this utility."
    echo " "
    echo "Please note that if you are manually uninstalling a WebSphere Application"
    echo "Server product, you must follow the manual uninstall instructions outlined in"
    echo "the WebSphere InfoCenter. This utility is provided to assist with that process."
    echo " "
    echo "This script will generate a log file named $LOGFILE"
    echo "The file will capture the output generated on the console. Please preserve"
    echo "this log file in case support representatives need to investigate this issue"
    echo "further."
    echo " "
    echo "Press ENTER to continue, or use ^C to quit now."
    read CONTINUE
}



# get_PRODDIR -
#  Get installation directory from user input, store it in PRODDIR
#
# Die if input is blank.

get_PRODDIR() {
    echo " "
    echo "Carefully enter the FULL path into which the product was installed."
    echo " "
    echo "- Do not include a \"/\" character at the end of the path."
    echo "- Do not use relative directoy addressing.  The path must start with a \"/\" and"
    echo "  \"..\" is not allowed."
    echo "- Enter the exact directory name as was input into the installation program,"
    echo "  not a symbolic link which references the same location."
    echo " "
    echo "Enter the input here:"
    read PRODDIR
    
    if [[ $PRODDIR = "" ]] then
        echo "Error: No input detected. Aborting."
        exit 1
    fi
}




# Call the main routine.

main
